@@ -3,19 +3,8 @@
 
 """Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from tile import Tile, elements
@@ -63,7 +52,7 @@ class RuleCode:
 class MJRule(RuleCode):
 
     def computeLastMelds(hand):
-        """returns all possible last melds"""
+        """return all possible last melds"""
 
 
 class DragonPungKong(RuleCode):
@@ -306,7 +295,7 @@ class StandardMahJongg(MJRule):
     cache = ('appliesToHand',)
 
     def computeLastMelds(hand):
-        """returns all possible last melds"""
+        """return all possible last melds"""
         return MeldList(x for x in hand.melds if hand.lastTile in x and len(x) < 4)
 
     def appliesToHand(hand):
@@ -342,9 +331,8 @@ class StandardMahJongg(MJRule):
             if val0 == 8:
                 return {Tile(group, val0 - 1)}
             return {Tile(group, val0 - 1), Tile(group, val0 + 2)}
-        else:
-            assert val0 + 2 == val1, 'group:%s values:%s' % (group, values)
-            return {Tile(group, val0 + 1)}
+        assert val0 + 2 == val1, 'group:%s values:%s' % (group, values)
+        return {Tile(group, val0 + 1)}
 
     def winningTileCandidates(cls, hand):
         # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches,too-many-statements
@@ -353,7 +341,7 @@ class StandardMahJongg(MJRule):
             return set()
         if not hand.tilesInHand:
             return set()
-        inHand = list(x.exposed for x in hand.tilesInHand)
+        inHand = [x.exposed for x in hand.tilesInHand]
         result = inHand[:]
         pairs = 0
         isolated = 0
@@ -430,7 +418,7 @@ class StandardMahJongg(MJRule):
             if len(values) % 3 == 0:
                 # adding a 4th, 7th or 10th tile with this color can not let us win,
                 # so we can exclude this color from the candidates
-                result = list(x for x in result if x.group != group)
+                result = [x for x in result if x.group != group]
                 continue
             valueSet = set(values)
             if len(values) == 4 and len(values) == len(valueSet):
@@ -448,9 +436,9 @@ class StandardMahJongg(MJRule):
             if len(valueSet) == 1:
                 # no chow reachable, only pair/pung
                 continue
-            singles = set(x for x in valueSet
-                          if values.count(x) == 1
-                          and not set([x - 1, x - 2, x + 1, x + 2]) & valueSet)
+            singles = {x for x in valueSet
+                       if values.count(x) == 1
+                       and not {x - 1, x - 2, x + 1, x + 2} & valueSet}
             isolated += len(singles)
             if isolated > 1:
                 # this is not a calling hand
@@ -514,14 +502,11 @@ class WrigglingSnake(MJRule):
     def shouldTry(hand, maxMissing=3):
         if hand.declaredMelds:
             return False
-        return (len(set(x.exposed for x in hand.tiles)) + maxMissing > 12
+        return (len({x.exposed for x in hand.tiles}) + maxMissing > 12
                 and all(not x.isChow for x in hand.declaredMelds))
 
     def computeLastMelds(hand):
-        if hand.lastTile.value == 1:
-            return [hand.lastTile.pair]
-        else:
-            return [hand.lastTile.single]
+        return [hand.lastTile.pair] if hand.lastTile.value == 1 else [hand.lastTile.single]
 
     def winningTileCandidates(hand):
         suits = hand.suits.copy()
@@ -534,17 +519,15 @@ class WrigglingSnake(MJRule):
         values = set(hand.values)
         if len(values) < 12:
             return set()
-        elif len(values) == 12:
+        if len(values) == 12:
             # one of 2..9 or a wind is missing
             if hand.values.count(1) < 2:
                 # and the pair of 1 is incomplete too
                 return set()
-            else:
-                return (elements.winds | set([Tile(group, x) for x in range(2, 10)])) \
-                    - set([x.exposed for x in hand.tiles])
-        else:
-            # pair of 1 is not complete
-            return set([Tile(group, '1')])
+            return (elements.winds | {Tile(group, x) for x in range(2, 10)}) \
+                - {x.exposed for x in hand.tiles}
+        # pair of 1 is not complete
+        return {Tile(group, '1')}
 
     def rearrange(hand, rest):
         melds = []
@@ -576,7 +559,7 @@ class CallingHand(RuleCode):
 
     def appliesToHand(cls, hand):
         for callHand in hand.callingHands:
-            used = list(x.rule.__class__ for x in callHand.usedRules)
+            used = (x.rule.__class__ for x in callHand.usedRules)
             if cls.limitHand in used:
                 return True
         return False
@@ -585,12 +568,12 @@ class CallingHand(RuleCode):
 class TripleKnitting(MJRule):
 
     def computeLastMelds(cls, hand):
-        """returns all possible last melds"""
+        """return all possible last melds"""
         if not hand.lastTile:
-            return
+            return None
         triples, rest = cls.findTriples(hand)
         assert len(rest) == 2
-        triples = list(triples)
+        triples = [triples]
         triples.append(rest)
         return [Meld(x) for x in triples if hand.lastTile in x]
 
@@ -622,7 +605,7 @@ class TripleKnitting(MJRule):
         while len(rest) >= 2:
             for tile in sorted(set(rest)):
                 value = tile.value
-                suits = set(x.group for x in rest if x.value == value)
+                suits = {x.group for x in rest if x.value == value}
                 if len(suits) < 2:
                     yield tuple(melds), tuple(rest)
                     return
@@ -652,7 +635,7 @@ class TripleKnitting(MJRule):
         if len(rest) not in (1, 4):
             return set()
         result = list(
-            [Tile(x, y.value).concealed for x in Tile.colors for y in rest])
+            Tile(x, y.value).concealed for x in Tile.colors for y in rest)
         for restTile in rest:
             result.remove(restTile)
         return set(result)
@@ -665,18 +648,15 @@ class TripleKnitting(MJRule):
         return tripleCount >= tripleWanted
 
     def findTriples(cls, hand):
-        """returns a list of triple knitted melds, including the mj triple.
+        """return a list of triple knitted melds, including the mj triple.
         Also returns the remaining untripled tiles"""
         if hand.declaredMelds:
             if len(hand.declaredMelds) > 1:
                 return (Meld(), None)
         result = []
-        tilesS = list(
-            x.concealed for x in hand.tiles if x.lowerGroup == Tile.stone)
-        tilesB = list(
-            x.concealed for x in hand.tiles if x.lowerGroup == Tile.bamboo)
-        tilesC = list(
-            x.concealed for x in hand.tiles if x.lowerGroup == Tile.character)
+        tilesS = [x.concealed for x in hand.tiles if x.lowerGroup == Tile.stone]
+        tilesB = [x.concealed for x in hand.tiles if x.lowerGroup == Tile.bamboo]
+        tilesC = [x.concealed for x in hand.tiles if x.lowerGroup == Tile.character]
         for tileS in tilesS[:]:
             tileB = Tile(Tile.bamboo, tileS.value).concealed
             tileC = Tile(Tile.character, tileS.value).concealed
@@ -691,7 +671,7 @@ class TripleKnitting(MJRule):
 class Knitting(MJRule):
 
     def computeLastMelds(cls, hand):
-        """returns all possible last melds"""
+        """return all possible last melds"""
         if not hand.lastTile:
             return []
         couples, rest = cls.findCouples(hand)
@@ -743,9 +723,9 @@ class Knitting(MJRule):
             return set()
         assert len(singleTile) == 1
         singleTile = singleTile[0]
-        otherSuit = (hand.suits - set([singleTile.lowerGroup])).pop()
+        otherSuit = (hand.suits - {singleTile.lowerGroup}).pop()
         otherTile = Tile(otherSuit, singleTile.value).concealed
-        return set([otherTile])
+        return {otherTile}
 
     def rearrange(cls, hand, rest):
         melds = []
@@ -759,7 +739,7 @@ class Knitting(MJRule):
         yield tuple(melds), tuple(rest)
 
     def findCouples(cls, hand, pairs=None):
-        """returns a list of tuples, including the mj couple.
+        """return a list of tuples, including the mj couple.
         Also returns the remaining uncoupled tiles IF they
         are of the wanted suits"""
         if hand.declaredMelds:
@@ -771,8 +751,8 @@ class Knitting(MJRule):
         suits = cls.pairSuits(hand)
         if not suits:
             return [], []
-        tiles0 = list(x for x in pairs if x.lowerGroup == suits[0])
-        tiles1 = list(x for x in pairs if x.lowerGroup == suits[1])
+        tiles0 = [x for x in pairs if x.lowerGroup == suits[0]]
+        tiles1 = [x for x in pairs if x.lowerGroup == suits[1]]
         for tile0 in tiles0[:]:
             if tile0.isExposed:
                 tile1 = Tile(suits[1], tile0.value)
@@ -785,15 +765,13 @@ class Knitting(MJRule):
         return result, tiles0 + tiles1
 
     def pairSuits(hand):
-        """returns a lowercase string with two suit characters. If no prevalence, returns None"""
-        suitCounts = list(len([x for x in hand.tiles if x.lowerGroup == y])
-                          for y in Tile.colors)
+        """return a lowercase string with two suit characters. If no prevalence, returns None"""
+        suitCounts = [len([x for x in hand.tiles if x.lowerGroup == y]) for y in Tile.colors]
         minSuit = min(suitCounts)
-        result = ''.join(
-            x for idx,
-            x in enumerate(Tile.colors) if suitCounts[idx] > minSuit)
+        result = ''.join(x for idx, x in enumerate(Tile.colors) if suitCounts[idx] > minSuit)
         if len(result) == 2:
             return result
+        return None
 
 
 class AllPairHonors(MJRule):
@@ -819,14 +797,12 @@ class AllPairHonors(MJRule):
             return False
         if len(set(hand.tiles)) != 7:
             return False
-        tileCounts = list([len([x for x in hand.tiles if x == y])
-                           for y in hand.tiles])
-        return set(tileCounts) == set([2])
+        return {len([x for x in hand.tiles if x == y]) for y in hand.tiles} == {2}
 
     def winningTileCandidates(cls, hand):
         if not cls.maybeCallingOrWon(hand):
             return set()
-        single = list(x for x in hand.tiles if hand.tiles.count(x) == 1)
+        single = [x for x in hand.tiles if hand.tiles.count(x) == 1]
         if len(single) != 1:
             return set()
         return set(single)
@@ -834,7 +810,7 @@ class AllPairHonors(MJRule):
     def shouldTry(hand, maxMissing=4):
         if hand.declaredMelds:
             return False
-        tiles = list(x.exposed for x in hand.tiles)
+        tiles = [x.exposed for x in hand.tiles]
         pairCount = kongCount = 0
         for tile in elements.majors:
             count = tiles.count(tile)
@@ -900,16 +876,14 @@ class BigFourJoys(RuleCode):
 class LittleFourJoys(RuleCode):
 
     def appliesToHand(hand):
-        lengths = sorted([min(len(x), 3) for x in hand.melds if x.isWindMeld])
+        lengths = sorted(min(len(x), 3) for x in hand.melds if x.isWindMeld)
         return lengths == [2, 3, 3, 3]
 
 
 class LittleThreeDragons(RuleCode):
 
     def appliesToHand(hand):
-        lengths = sorted([min(len(x), 3)
-                          for x in hand.melds if x.isDragonMeld])
-        return lengths == [2, 3, 3]
+        return sorted(min(len(x), 3) for x in hand.melds if x.isDragonMeld) == [2, 3, 3]
 
 
 class FourBlessingsHoveringOverTheDoor(RuleCode):
@@ -921,7 +895,7 @@ class FourBlessingsHoveringOverTheDoor(RuleCode):
 class AllGreen(RuleCode):
 
     def appliesToHand(hand):
-        return set(x.exposed for x in hand.tiles) < elements.greenHandTiles
+        return {x.exposed for x in hand.tiles} < elements.greenHandTiles
 
 
 class LastTileFromWall(RuleCode):
@@ -1038,8 +1012,7 @@ class GatesOfHeaven(StandardMahJongg):
             return False
         for suit in Tile.colors:
             count19 = sum(x.isTerminal for x in hand.tiles)
-            suitCount = len(
-                list(x for x in hand.tiles if x.lowerGroup == suit))
+            suitCount = len([x for x in hand.tiles if x.lowerGroup == suit])
             if suitCount > 10 and count19 > 4:
                 return True
         return False
@@ -1169,7 +1142,7 @@ class ThirteenOrphans(MJRule):
         return result
 
     def appliesToHand(hand):
-        return set(x.exposed for x in hand.tiles) == elements.majors
+        return {x.exposed for x in hand.tiles} == elements.majors
 
     def winningTileCandidates(cls, hand):
         if any(x in hand.values for x in Tile.minors):
@@ -1177,20 +1150,19 @@ class ThirteenOrphans(MJRule):
             return set()
         if not cls.shouldTry(hand, 1):
             return set()
-        handTiles = set(x.exposed for x in hand.tiles)
+        handTiles = {x.exposed for x in hand.tiles}
         missing = elements.majors - handTiles
-        if len(missing) == 0:
+        if not missing:
             # if all 13 tiles are there, we need any one of them:
             return elements.majors
-        else:
-            assert len(missing) == 1
-            return missing
+        assert len(missing) == 1
+        return missing
 
     def shouldTry(hand, maxMissing=4):
         # TODO: look at how many tiles there still are on the wall
         if hand.declaredMelds:
             return False
-        handTiles = set(x.exposed for x in hand.tiles)
+        handTiles = {x.exposed for x in hand.tiles}
         missing = elements.majors - handTiles
         if len(missing) > maxMissing:
             return False
@@ -1203,7 +1175,7 @@ class ThirteenOrphans(MJRule):
         hand = candidates.hand
         if not cls.shouldTry(hand):
             return candidates
-        handTiles = set(x.exposed for x in hand.tiles)
+        handTiles = {x.exposed for x in hand.tiles}
         missing = elements.majors - handTiles
         havePair = False
         keep = (6 - len(missing)) * 5
@@ -1310,7 +1282,8 @@ class BlessingOfHeaven(RuleCode):
             return False
         if hand.ownWind is not East:
             return False
-        assert hand.lastTile.isConcealed, '{}: Blessing of Heaven: last tile must be concealed'.format(hand)
+        assert not any(x.isExposed for x in hand.melds)
+        assert hand.lastTile is None, '{}: Blessing of Heaven: There can be no last tile'.format(hand)
         return True
 
     def selectable(hand):
@@ -1339,7 +1312,7 @@ class BlessingOfEarth(RuleCode):
 class LongHand(RuleCode):
 
     def appliesToHand(hand):
-        return (not hand.won and hand.lenOffset > 0) or hand.lenOffset > 1
+        return hand.lenOffset > 0 if not hand.won else hand.lenOffset > 1
 
 
 class FalseDiscardForMJ(RuleCode):
